package de.gc.streams.komp;

import java.util.Map;
import java.util.stream.Collectors;

import de.gc.model.Ort;
import de.gc.model.PlzList;
import de.gc.util.Util;

public class Lamda_12_groupby_04 {

   // @formatter:off
   public static class BundeslandStats {
      private final long anzahlOrte;
      private final int summeEinwohner;

      public BundeslandStats(final long anzahlOrte, final int summeEinwohner) {
         this.anzahlOrte = anzahlOrte;
         this.summeEinwohner = summeEinwohner;
      }

      public long getAnzahlOrte() {
         return anzahlOrte;
      }

      public int getSummeEinwohner() {
         return summeEinwohner;
      }
   }

   public static void main(final String[] args) {
      final PlzList plzListe = new PlzList("/plz_orte.csv");

      // @formatter:off
      final long start = Util.getStartTime();
      // Ermitteln der Anzahl der Orte und der Summe der Einwohner pro Bundesland
      final Map<String, BundeslandStats> anzahlUndEinwohnerProBundesland = plzListe
        .parallelStream().unordered()
        .collect(Collectors.groupingByConcurrent(
          Ort::getBundesland,
          Collectors.collectingAndThen(Collectors.toList(),
          list -> {
            final long anzahlOrte = list.size();
            final int summeEinwohner = list.stream().mapToInt(Ort::getEinwohnerzahl).sum();
            return new BundeslandStats(anzahlOrte, summeEinwohner);
      })));

      // Ausgabe der Ergebnisse
      anzahlUndEinwohnerProBundesland.forEach((bundesland, stats)
            -> System.out.format("%s : %d Orte, Summe Einwohner: %d%n",
                  bundesland, stats.getAnzahlOrte(), stats.getSummeEinwohner()));

      System.out.println("#######################");
      Util.printTime(start);
      // @formatter:on
   }
}