package de.gc.streams.komp;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import de.gc.model.Ort;
import de.gc.model.PlzList;
import de.gc.util.Util;

public class Lamda_13_groupby_05 {

   public static class LandkreisStats {
      private final long anzahlOrte;
      private final double durchschnittEinwohner;
      private final List<Ort> orte;

      public LandkreisStats(final long anzahlOrte, final double durchschnittEinwohner, final List<Ort> orte) {
         this.anzahlOrte = anzahlOrte;
         this.durchschnittEinwohner = durchschnittEinwohner;
         this.orte = orte;
      }

      public long getAnzahlOrte() {
         return anzahlOrte;
      }

      public double getDurchschnittEinwohner() {
         return durchschnittEinwohner;
      }

      public List<Ort> getOrte() {
         return orte;
      }
   }

   public static void main(final String[] args) {
      final PlzList plzListe = new PlzList("/plz_orte.csv");

      // @formatter:off
      final long start = Util.getStartTime();
      // Filtere nach Bundesland "Bayern", gruppiere nach Landkreis und berechne Statistiken
      final Map<String, LandkreisStats> landkreiseStats = plzListe.parallelStream()
          .unordered()
          .filter(ort -> "Bayern".equals(ort.getBundesland())) // Filter für "Bayern"
          .collect(Collectors.groupingByConcurrent(
              Ort::getLandkreis,
              Collectors.collectingAndThen(
                  Collectors.toList(),
                  list -> {
                      final long anzahlOrte = list.size();
                      final double durchschnittEinwohner = list.stream()
                          .mapToInt(Ort::getEinwohnerzahl)
                          .average()
                          .orElse(0.0);
                      return new LandkreisStats(anzahlOrte, durchschnittEinwohner, list);
                  }
              )
          ));

      // Ausgabe der Statistiken für jeden Landkreis
      landkreiseStats.forEach((landkreis, stats) -> {
          System.out.format("Landkreis: %s%n", landkreis);
          System.out.format("Anzahl der Orte: %d%n", stats.getAnzahlOrte());
          System.out.format("Durchschnittliche Einwohnerzahl: %.2f%n",
                stats.getDurchschnittEinwohner());

          // Ausgabe der Orte und ihrer Einwohnerzahlen
          stats.getOrte().forEach(ort ->
              System.out.format("Ort: %s, PLZ: %s, Einwohnerzahl: %d%n",
                  ort.getOrtsname(), ort.getPlz(), ort.getEinwohnerzahl()));
      });
      // @formatter:on

      System.out.println("#######################");
      Util.printTime(start);
   }
}