package de.gc.streams.komp;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import de.gc.model.Ort;
import de.gc.model.PlzList;
import de.gc.util.Util;

public class Lamda_14_groupby_06 {

   public static void main(final String[] args) {
      final PlzList plzListe = new PlzList("/plz_orte.csv");

      // @formatter:off
      final long start = Util.getStartTime();
      // Erstelle die Map, die für jedes Bundesland die Summe der Einwohner und
      // die Orte mit weniger als 1000 Einwohner enthält
      final Map<String, Map<String, Object>> ergebnisMap = plzListe.parallelStream()
          .unordered()
          .collect(Collectors.groupingByConcurrent(
              Ort::getBundesland,
              Collectors.collectingAndThen(
                  Collectors.toList(),
                  liste -> {
                      // Berechne die Summe der Einwohner
                      final long summeEinwohner = liste.stream()
                          .mapToLong(Ort::getEinwohnerzahl)
                          .sum();

                      // Filtere Orte mit weniger als 1000 Einwohnern und limitiere auf 10 Orte
                      final List<Ort> gefilterteOrte = liste.stream()
                          .filter(ort -> ort.getEinwohnerzahl() < 1000)
                          .sorted(Comparator.comparingInt(Ort::getEinwohnerzahl))
                          .limit(10)
                          .collect(Collectors.toList());

                      // Erstelle eine Map für die Ergebnisse
                      final Map<String, Object> resultMap = new HashMap<>();
                      resultMap.put("SummeEinwohner", summeEinwohner);
                      resultMap.put("Orte", gefilterteOrte);
                      return resultMap;
                  }
              )
          ));

      // Sortiere die Map nach Bundeslandnamen
      final List<String> sortedBundesländer = new ArrayList<>(ergebnisMap.keySet());
      sortedBundesländer.sort(String::compareTo);

      // Ausgabe der Ergebnisse
      sortedBundesländer.forEach(bundesland -> {
          final Map<String, Object> resultMap = ergebnisMap.get(bundesland);
          final long summeEinwohner = (Long) resultMap.get("SummeEinwohner");
          final List<Ort> gefilterteOrte = (List<Ort>) resultMap.get("Orte");

          // Ausgabe der Summe der Einwohner
          System.out.format("Bundesland: %s, Summe Einwohner: %d%n", bundesland, summeEinwohner);

          // Ausgabe der Orte mit weniger als 1000 Einwohnern, max. 10 Orte
          if (!gefilterteOrte.isEmpty()) {
              System.out.println("Orte mit weniger als 1000 Einwohner:");
              gefilterteOrte.forEach(ort ->
                  System.out.format("Ort: %s, PLZ: %s, Einwohnerzahl: %d%n",
                      ort.getOrtsname(), ort.getPlz(), ort.getEinwohnerzahl())
              );
          }
          System.out.println();
      });
      // @formatter:on

      System.out.println("#######################");
      Util.printTime(start);
   }
}