package de.gc.util;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;

import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVPrinter;
import org.apache.commons.csv.CSVRecord;
import org.apache.commons.csv.QuoteMode;

import de.gc.model.Ort;
import de.gc.model.PlzList;

public class Convert {

   private static final String PLZ_ORTE_CSV = "src/main/resources/plz_orte.csv";

   public static void main(final String[] args) {
      final Convert convert = new Convert();
      try {
         final PlzList plzList = convert.convertCsvToPlzList("/plz_einwohner.csv", "/zuordnung_plz_ort.csv");
         plzList.forEach(System.out::println);
         convert.writeToCsv(plzList, PLZ_ORTE_CSV);
         System.out.format("Anzahl Orte: %d - in Datei %s gespeichert%n", plzList.size(), PLZ_ORTE_CSV);
      } catch (final IOException e) {
         e.printStackTrace();
      }

   }

   public PlzList convertCsvToPlzList(final String plzEinwohnerCsv, final String zuordnungPlzOrtCsv)
         throws IOException {

      final PlzList plzList = new PlzList();

      // Teil 1 --------------------
      final BufferedReader plzEinReader = new BufferedReader(
            new InputStreamReader(getClass().getResourceAsStream(plzEinwohnerCsv)));
      final HashMap<String, Integer> plzEinMap = new HashMap<String, Integer>();

      Iterable<CSVRecord> records = CSVFormat.EXCEL.builder().setDelimiter(',').setQuote('"').setIgnoreEmptyLines(true)
            .setIgnoreSurroundingSpaces(true).setHeader().setSkipHeaderRecord(true).build().parse(plzEinReader);
      for (final CSVRecord record : records) {
         final String plz = record.get(0);
         final String einwohner = record.get(2);
         // System.out.println(plz + " " + einwohner);
         int ieinwohner = 0;
         try {
            ieinwohner = Integer.parseInt(einwohner.trim());
         } catch (final NumberFormatException e) {
            System.out.println("Einwohnerzahl konnte nicht in Integer umgewandelt werden: " + einwohner);
         }
         plzEinMap.put(plz, ieinwohner);
      }
      plzEinReader.close();

      // Teil 2 --------------------
      final BufferedReader plzOrtReader = new BufferedReader(
            new InputStreamReader(getClass().getResourceAsStream(zuordnungPlzOrtCsv)));
      records = CSVFormat.EXCEL.builder().setDelimiter(',').setQuote('"').setIgnoreEmptyLines(true)
            .setIgnoreSurroundingSpaces(true).setHeader().setSkipHeaderRecord(true).build().parse(plzOrtReader);
      for (final CSVRecord record : records) {
         final String ort = record.get(2).trim();
         final String plz = record.get(3).trim();
         final String landkreis = record.get(4).trim();
         final String bundesland = record.get(5).trim();
         int ieinwohner = 0;
         if (plzEinMap.containsKey(plz)) {
            ieinwohner = plzEinMap.get(plz);
         }
         final Ort o = new Ort(ort, plz, landkreis, bundesland, ieinwohner);
         plzList.add(o);
         // System.out.println(o);
      }
      plzOrtReader.close();

      return plzList;
   }

   private void writeToCsv(final PlzList plzList, final String filename) {

      try (BufferedWriter w = new BufferedWriter(new FileWriter(filename))) {

         final CSVPrinter csvPrinter = new CSVPrinter(w,
               CSVFormat.EXCEL.builder().setDelimiter(";").setQuote('"').setQuoteMode(QuoteMode.ALL)
                     .setHeader(new String[] { "Ortsname", "PLZ", "Landkreis", "Bundesland", "Einwohnerzahl" })
                     .build());

         for (final Ort o : plzList) {
            csvPrinter.printRecord(o.getOrtsname(), o.getPlz(), o.getLandkreis(), o.getBundesland(),
                  o.getEinwohnerzahl());
         }
         csvPrinter.close();
         w.close();

      } catch (final IOException e) {
         e.printStackTrace();
      }
   }
}
